/**
 * http://valileo-valilei.blogspot.de/2010/09/getting-link-local-addres-from.html
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <sys/ioctl.h>

#include <net/if.h>
#include <netinet/if_ether.h>

#include <sys/types.h>
#include <ifaddrs.h>

int get_link_local_addr(char* if_name, int if_name_length, struct sockaddr_in6 *ip)
{
	struct ifaddrs *ifaddr, *ifa;
	char test[200];
	
	if (getifaddrs(&ifaddr) == -1) {
	    fprintf(stderr, "getifaddrs failed");
		freeifaddrs(ifaddr);
		return 1;
	}
	
	for (ifa = ifaddr; ifa != NULL; ifa = ifa->ifa_next)
	{
		if (strncmp(ifa->ifa_name, if_name, if_name_length)) continue;
		
		struct sockaddr_in6 *current_addr = (struct sockaddr_in6 *) ifa->ifa_addr;
		
		if (ifa->ifa_addr->sa_family == AF_INET) {
			inet_ntop(AF_INET, &(((struct sockaddr_in *)current_addr)->sin_addr), test, 200);
			printf(" ipv4 %s", test);
		}
		
		if (ifa->ifa_addr->sa_family == AF_INET6) {
			if (IN6_IS_ADDR_LINKLOCAL(&(current_addr->sin6_addr))) {
				inet_ntop(AF_INET6, &(((struct sockaddr_in6 *)current_addr)->sin6_addr), test, 200);
				printf(" local ipv6 %s", test);
			} else {
				inet_ntop(AF_INET6, &(((struct sockaddr_in6 *)current_addr)->sin6_addr), test, 200);
				printf(" non local iv6 %s", test);
			}
		}
//		memcpy(ip, current_addr, sizeof(*current_addr));
	}
	
	freeifaddrs(ifaddr);
	return 0;
}
 
/* 
 * this is straight from beej's network tutorial. It is a nice wrapper
 * for inet_ntop and helpes to make the program IPv6 ready
 */
char *get_ip_str(const struct sockaddr *sa, char *s, size_t maxlen)
{
  switch(sa->sa_family) {
  case AF_INET:
    inet_ntop(AF_INET, &(((struct sockaddr_in *)sa)->sin_addr),
	      s, maxlen);
    break;
 
  case AF_INET6:
    inet_ntop(AF_INET6, &(((struct sockaddr_in6 *)sa)->sin6_addr),
	      s, maxlen);
    break;
 
  default:
    strncpy(s, "Unknown AF", maxlen);
    return NULL;
  }

  return s;
}

int main(void)
{
	char            buf[8192] = {0};
	struct ifconf   ifc = {0};
	struct ifreq   *ifr = NULL;
	int             sck = 0;
	int             i = 0;
	char            ip[INET6_ADDRSTRLEN] = {0};
	struct ifreq    *item;
	char            hostname[NI_MAXHOST];
	struct sockaddr_in6 *ipv6address;
	
	/* Get a socket handle. */
	sck = socket(PF_INET, SOCK_DGRAM, 0);
	if(sck < 0) {
		fprintf(stderr, "getting socket handle failed");
		return 1;
	}
	
	/* Query available interfaces. */
	ifc.ifc_len = sizeof(buf);
	ifc.ifc_buf = buf;
	if(ioctl(sck, SIOCGIFCONF, &ifc) < 0) {
		fprintf(stderr, "ioctl(SIOCGIFCONF) failed");
		return 1;
	}
	
	/* Iterate through the list of interfaces. */
	ifr = ifc.ifc_req;
	for(i = 0; i < (ifc.ifc_len / sizeof(struct ifreq)); i++) {
		bzero(hostname, NI_MAXHOST);
		item = &ifr[i];
		
		/* Get the address 
		* This may seem silly but it seems to be needed on some systems 
		*/
		if(ioctl(sck, SIOCGIFADDR, item) < 0) {
			fprintf(stderr, "ioctl(OSIOCGIFADDR) failed");
		}
		printf("%s %s", item->ifr_name, get_ip_str(&item->ifr_addr, ip, INET6_ADDRSTRLEN));
		
		/* ipv4 netmask */
		if(ioctl(sck, SIOCGIFNETMASK, item) < 0) {
			fprintf(stderr, "ioctl(SIOCGIFNETMASK) failed");
			return 1;
		}
		printf(" SIOCGIFNETMASK %s", inet_ntoa(((struct sockaddr_in *)&item->ifr_netmask)->sin_addr));
	
		/* get ipv6 addresses */
		get_link_local_addr(item->ifr_name, strlen(item->ifr_name), ipv6address);
		
		/* Get the MAC address */
		if(ioctl(sck, SIOCGIFHWADDR, item) < 0) {
			fprintf(stderr, "ioctl(SIOCGIFHWADDR) failed");
			return 1;
		}
		
		/* display result */
		printf(" %02x:%02x:%02x:%02x:%02x:%02x",
			(unsigned char)item->ifr_hwaddr.sa_data[0],
			(unsigned char)item->ifr_hwaddr.sa_data[1],
			(unsigned char)item->ifr_hwaddr.sa_data[2],
			(unsigned char)item->ifr_hwaddr.sa_data[3],
			(unsigned char)item->ifr_hwaddr.sa_data[4],
			(unsigned char)item->ifr_hwaddr.sa_data[5]);
		
		if(ioctl(sck, SIOCGIFBRDADDR, item) < 0) {
			fprintf(stderr, "ioctl(SIOCGIFBRDADDR) failed");
			return 1;
		}
		printf(" BROADCAST %s", inet_ntoa(((struct sockaddr_in *)&item->ifr_broadaddr)->sin_addr));
		
		if(ioctl(sck, SIOCGIFMTU, item) < 0) {
			fprintf(stderr, "ioctl(SIOCGIFMTU) failed");
			return 1;
		}
		printf(" MTU %d", item->ifr_mtu);
		
		printf("\n");
	}
	return 0;
}